#include "DirectXLib/DirectXLib.h"
#include "Camera.h"


#if false
struct CameraData
{
	D3DXVECTOR3	pos;
	D3DXVECTOR3	taget;
	float		speed;
	float		dist;
	float		distBase;
	float		distNear;
	float		distFar;
	float		height;
	D3DXMATRIX	view;
	D3DXMATRIX	projection;
};
static CameraData g_cameraData;


// J̏
void InitializeCamera(D3DXVECTOR3 pos, D3DXVECTOR3 target)
{
	ZeroMemory(&g_cameraData, sizeof(g_cameraData));
	D3DXMatrixIdentity(&g_cameraData.view);
	D3DXMatrixIdentity(&g_cameraData.projection);

	g_cameraData.pos	= pos;
	g_cameraData.taget	= target;
	SetCameraProjection(D3DXToRadian(45.0f));
	SetCameraSpeed(0.25f);
	SetCameraDistance(6.5f, 5.0f, 10.0f);
	SetCameraAddHeight(2.0f);
}

// J̍XV
void UpdateCamera(const D3DXVECTOR3* pTarget)
{
	const float Speed		= g_cameraData.speed;
	const float BaseDist	= g_cameraData.distBase;
	const float NearDist	= g_cameraData.distNear;
	const float FarDist		= g_cameraData.distFar;

	D3DXVECTOR3 cameraPos(g_cameraData.pos);
	D3DXVECTOR3 targetPos(*pTarget);
	D3DXVECTOR3 cameraVec(targetPos - cameraPos);

	cameraVec.y = 0.0f;

	float dist = D3DXVec3Length(&cameraVec);
	// K
	D3DXVec3Scale(&cameraVec, &cameraVec, 1.0f / dist);

	D3DXVECTOR3 nextPos;
	// J̗zʒuvZ
	nextPos = targetPos - cameraVec * g_cameraData.distBase;
	nextPos.y += g_cameraData.height;

	// {ړl
	D3DXVECTOR3 distVec(nextPos - cameraPos);
	D3DXVec3Normalize(&distVec, &distVec);

	// J王_܂ł̋ɂăJ̑xɋ
	float moveDist = D3DXVec3Length(&(cameraPos - targetPos));
	moveDist -= BaseDist;
	if( moveDist < 0.0f )	moveDist /= (NearDist - BaseDist);
	else					moveDist /= (FarDist - BaseDist);
	moveDist *= moveDist;

	// J̈ʒuZo
	cameraPos += distVec * moveDist * Speed;

	g_cameraData.pos	= cameraPos;
	g_cameraData.taget	= targetPos;

	D3DXVECTOR3 upVec(0.0f, 1.0f, 0.0f);
	// Js̍쐬
	D3DXMatrixLookAtLH(&g_cameraData.view, &cameraPos, &targetPos, &upVec);
}

// Projection̐ݒ
void SetCameraProjection(float fov, float znear, float zfar)
{
	SimpleWindow* pWindow = DirectXLib::Instance.GetWindow();
	D3DXMatrixPerspectiveFovLH(
		&g_cameraData.projection,
		fov,
		(float)pWindow->GetWidth() / (float)pWindow->GetHeight(),
		znear,
		zfar
	);
}

// J̕ωxݒ
void SetCameraSpeed(float speed)
{
	g_cameraData.speed = speed;
}

// ^[QbgƂ̋ݒ
void SetCameraDistance(float dist, float distNear, float distFar)
{
	g_cameraData.distBase	= dist;
	g_cameraData.distNear	= distNear;
	g_cameraData.distFar	= distFar;
}

// ^[QbgƂ̋ݒ
void SetCameraAddHeight(float height) { g_cameraData.height = height; }

// JW̎擾
D3DXVECTOR3 GetCameraPosition(void) { return g_cameraData.pos; }
// J^[Qbg̎擾
D3DXVECTOR3 GetCameraTarget(void) { return g_cameraData.taget; }

// Views̎擾
const D3DXMATRIX* GetViewMatrix(void) { return &g_cameraData.view; }
// Projections̎擾
const D3DXMATRIX* GetProjectionMatrix(void) { return &g_cameraData.projection; }

#endif