#pragma once

#include "../DirectXLib.h"
#include <fbxsdk.h>
#include <vector>
#include <map>
#include <string>


static const int MOTION_MAX = 256; 
static const int BONE_MAX = 256; 

// {[
struct Bone
{
	char		name[64];
	D3DXMATRIX	offset;
	D3DXMATRIX	transform;
};

//	Aj[V
struct Motion
{
	Motion(){}

	UINT					numFrame;		// t[
	std::vector<D3DXMATRIX> Key[BONE_MAX];	// L[t[
};


// FBXfNX
class FbxModel
	: public Model
{
public:
	FbxModel(void);
	~FbxModel(void);

	// FBXbV̓ǂݍ
	bool Load(const char* pFilename);
	// FBXbV̉
	void Finalize(void);

	// FBXbV̕`
	void Render(void);


	// Aj[V̍Đ
	void Play(std::string name);
	// Aj[V̍XV
	void Animate(float sec = 1.0f / 60.0f);

private:
	void Skinning(void);

	// FBX̉
	bool LoadMesh(UINT meshIdx, FbxMesh* pMesh);
	void LoadIndeces(UINT meshIdx, FbxMesh* pMesh);
	void LoadPosition(UINT meshIdx, FbxMesh* pMesh);
	void LoadNormal(UINT meshIdx, FbxMesh* pMesh);
	void LoadUV(UINT meshIdx, FbxMesh* pMesh);
	void LoadColor(UINT meshIdx, FbxMesh* pMesh);
	void LoadMaterial(UINT matIdx, FbxMesh* pMesh);

	int FindBone(const char* pName);
	void LoadBone(UINT matIdx, FbxMesh* pMesh);
	void LoadKeyFrames(std::string name, int bone, FbxNode* pBoneNode);
	void LoadMotion(std::string name, const char* pFilename);

private:
	char							m_rootDir[MAX_PATH];	// [gpX
	std::string						m_playMotion;			// [V
	float							m_frame;				// t[
	int								m_startFrame;			// Jnt[
	UINT							m_boneNum;				// {[
	Bone							m_bone[BONE_MAX];		// {[
	std::map<std::string, Motion>	m_motion;				// [V
	std::vector<MeshVertex*>		m_pVertices;			// _f[^
};
