#include "DirectXLib/DirectXLib.h"
#include "Helper/DDSHelpter.h"
#include <stdio.h>
#include <vector>
#include <DDSTextureLoader.h>
#include <ResourceUploadBatch.h>

#define	TEXTURE_MAX	1000

// eNX`[f[^
struct TextureData
{
	UINT				width;		// eNX`[̕
	UINT				height;		// eNX`[̍
	UINT				srcWidth;	// 摜̕
	UINT				srcHeight;	// 摜̍
	ComPtr<ID3D12Resource> pTexture;// eNX`[̃|C^
};

struct LoadedTextureInfo
{
	char		filename[MAX_PATH];
	int			count;
	TextureData	texData;
};
static LoadedTextureInfo g_loadedTextures[TEXTURE_MAX] = {};

// eNX`[̓ǂݍ
static const TextureData* LoadTexture(const char* pFileName)
{
	for( int i=0; i < TEXTURE_MAX; i++ )
	{
		LoadedTextureInfo* pInfo = &g_loadedTextures[i];
		if( pInfo->count == 0 ) continue;

		if( strcmp(pInfo->filename, pFileName) == 0 )
		{
			pInfo->count ++;
			return &pInfo->texData;
		}
	}

	for (int i = 0; i < TEXTURE_MAX; i++)
	{
		LoadedTextureInfo* pInfo = &g_loadedTextures[i];
		if (pInfo->count != 0) continue;

		FILE* hFile = fopen(pFileName, "rb");
		if (hFile == nullptr) {
			return nullptr;
		}
		fseek(hFile, 0, SEEK_END);
		int fileSize = ftell(hFile);
		fseek(hFile, 0, SEEK_SET);

		uint8_t* pBuffer = (uint8_t*)malloc(fileSize);
		fread(pBuffer, fileSize, 1, hFile);
		fclose(hFile);

#if true
		DDSAnalyzer ddsAnalyzer;
		if (!ddsAnalyzer.Analyze(pBuffer))
		{
			free(pBuffer);
			return NULL;
		}

		auto pDirectX = DirectXLib::Instance.GetDirectX();
		auto& pDevice = pDirectX->GetDevice();
		auto& pCommandList = pDirectX->GetCommandList();
		auto& descripterHeap = pDirectX->GetGraphicsDescripterHeap();
		auto& pHeap = descripterHeap.GetHeap();

		// Note: ComPtr's are CPU objects but this resource needs to stay in scope until
		// the command list that references it has finished executing on the GPU.
		// We will flush the GPU at the end of this method to ensure the resource is not
		// prematurely destroyed.
		ComPtr<ID3D12Resource> textureUploadHeap;

		UINT mipLevels = 1;// ddsAnalyzer.MipLevels();
		// Describe and create a Texture2D.
		D3D12_RESOURCE_DESC textureDesc{};
		textureDesc.MipLevels = mipLevels;
		textureDesc.Format = DXGI_FORMAT_BC3_UNORM;
		textureDesc.Width = ddsAnalyzer.Width();
		textureDesc.Height = ddsAnalyzer.Height();
		textureDesc.Flags = D3D12_RESOURCE_FLAG_NONE;
		textureDesc.DepthOrArraySize = 1;
		textureDesc.SampleDesc.Count = 1;
		textureDesc.SampleDesc.Quality = 0;
		textureDesc.Dimension = D3D12_RESOURCE_DIMENSION_TEXTURE2D;

		D3D12_HEAP_PROPERTIES prop{};
		prop.Type = D3D12_HEAP_TYPE_DEFAULT;
		prop.CPUPageProperty = D3D12_CPU_PAGE_PROPERTY_UNKNOWN;
		prop.MemoryPoolPreference = D3D12_MEMORY_POOL_UNKNOWN;
		prop.CreationNodeMask = 1;
		prop.VisibleNodeMask = 1;

		ComPtr<ID3D12Resource> pTexture;
		if (FAILED(pDevice->CreateCommittedResource(
			&prop,
			D3D12_HEAP_FLAG_NONE,
			&textureDesc,
			D3D12_RESOURCE_STATE_COPY_DEST,
			nullptr,
			IID_PPV_ARGS(&pTexture))))
		{
			return false;
		}
		auto desc = pTexture->GetDesc();

#if true
		UINT64 uploadBufferSize;
		D3D12_PLACED_SUBRESOURCE_FOOTPRINT layout;
		pDevice->GetCopyableFootprints(&textureDesc, 0, mipLevels, 0, &layout, nullptr, nullptr, &uploadBufferSize);

		prop.Type = D3D12_HEAP_TYPE_UPLOAD;

		D3D12_RESOURCE_DESC resourceDesc;
		resourceDesc.Dimension = D3D12_RESOURCE_DIMENSION_BUFFER;
		resourceDesc.Alignment = 0;
		resourceDesc.Width = uploadBufferSize;
		resourceDesc.Height = 1;
		resourceDesc.DepthOrArraySize = 1;
		resourceDesc.MipLevels = 1;
		resourceDesc.Format = DXGI_FORMAT_UNKNOWN;
		resourceDesc.SampleDesc.Count = 1;
		resourceDesc.SampleDesc.Quality = 0;
		resourceDesc.Layout = D3D12_TEXTURE_LAYOUT_ROW_MAJOR;
		resourceDesc.Flags = D3D12_RESOURCE_FLAG_NONE;
		// Create the GPU upload buffer.
		if (FAILED(pDevice->CreateCommittedResource(
			&prop,
			D3D12_HEAP_FLAG_NONE,
			&resourceDesc,
			D3D12_RESOURCE_STATE_GENERIC_READ,
			nullptr,
			IID_PPV_ARGS(&textureUploadHeap))))
		{
			return false;
		}
		uint8_t* pBits;

		textureUploadHeap->Map(0, nullptr, (void**)&pBits);

		UINT width = ddsAnalyzer.Width();
		UINT height = ddsAnalyzer.Height();
		UINT offset = 0;
		const char* pOffset = (const char*)ddsAnalyzer.PixelData();
		for (int i = 0; i < mipLevels; i++)
		{
			UINT pitch = 0;
			UINT size = 0;
			switch (ddsAnalyzer.Format())
			{
			case DDSFMT_A8R8G8B8:
			{
				UINT w = max(width >> i, 1U);
				UINT h = max(height >> i, 1U);
				size = h * w * 4;
			} break;

			case DDSFMT_DXT1:
			case DDSFMT_DXT3:
			case DDSFMT_DXT5:
			{
				UINT uiBlockSize = ddsAnalyzer.GetForatBlockSize();
				UINT w = max(width >> i, 1U);
				UINT h = max(height >> i, 1U);
				pitch = ALIGN(w, 4U) / 4 * uiBlockSize;
				size = ALIGN(h, 4U) / 4 * pitch;
			} break;
			}

			const UINT rows = size / pitch;
			for (UINT row = 0; row < rows; row++)
			{
				memcpy(pBits + layout.Footprint.RowPitch * row, pOffset + offset + pitch * row, pitch);
			}
			offset += size;
		}
		textureUploadHeap->Unmap(0, nullptr);

		D3D12_TEXTURE_COPY_LOCATION src{};
		src.pResource = textureUploadHeap.Get();
		src.Type = D3D12_TEXTURE_COPY_TYPE_PLACED_FOOTPRINT;
		src.PlacedFootprint = layout;
		D3D12_TEXTURE_COPY_LOCATION dst{};
		dst.pResource = pTexture.Get();
		dst.Type = D3D12_TEXTURE_COPY_TYPE_SUBRESOURCE_INDEX;
		dst.SubresourceIndex = 0;

		pDirectX->BeginCommand();
		pCommandList->CopyTextureRegion(&dst, 0, 0, 0, &src, nullptr);
		pDirectX->TranslationBarrier(pTexture.Get(), D3D12_RESOURCE_STATE_COPY_DEST, D3D12_RESOURCE_STATE_PIXEL_SHADER_RESOURCE);

		pCommandList->Close();
		pDirectX->ExecuteCommand();
		pDirectX->WaitCommand();
#else
		std::vector<D3D12_SUBRESOURCE_DATA> subresources;

		UINT width = ddsAnalyzer.Width();
		UINT height = ddsAnalyzer.Height();
		UINT offset = 0;
		const char* pOffset = (const char*)ddsAnalyzer.PixelData();
		for (int i = 0; i < mipLevels; i++)
		{
			UINT pitch = 0;
			UINT size = 0;
			switch (ddsAnalyzer.Format())
			{
			case DDSFMT_A8R8G8B8:
			{
				UINT w = max(width >> i, 1U);
				UINT h = max(height >> i, 1U);
				size = h * w * 4;
			} break;

			case DDSFMT_DXT1:
			case DDSFMT_DXT3:
			case DDSFMT_DXT5:
			{
				UINT uiBlockSize = ddsAnalyzer.GetForatBlockSize();
				UINT w = max(width >> i, 1U);
				UINT h = max(height >> i, 1U);
				pitch = ALIGN(w, 4U) / 4 * uiBlockSize;
				size = ALIGN(h, 4U) / 4 * pitch;
			} break;
			}

			D3D12_SUBRESOURCE_DATA data;
			data.pData = pOffset + offset;
			data.RowPitch = pitch;
			data.SlicePitch = size;
			subresources.push_back(data);
			offset += size;
		}
		DirectX::ResourceUploadBatch batch(pDevice.Get());

		batch.Begin();
		batch.Upload(pTexture.Get(), 0, subresources.data(), subresources.size());
		batch.Transition(pTexture.Get(), D3D12_RESOURCE_STATE_COPY_DEST, D3D12_RESOURCE_STATE_PIXEL_SHADER_RESOURCE);

		batch.End(pDirectX->GetCommandQueue().Get()).wait();
#endif
#else
		auto pDirectX = DirectXLib::Instance.GetDirectX();
		auto& pDevice = pDirectX->GetDevice();
		auto& pCommandList = pDirectX->GetCommandList();
		auto& descripterHeap = pDirectX->GetGraphicsDescripterHeap();
		auto& pHeap = descripterHeap.GetHeap();

		DirectX::ResourceUploadBatch batch(pDevice.Get());

		batch.Begin();
		ComPtr<ID3D12Resource> pTexture;
		std::vector<D3D12_SUBRESOURCE_DATA> subresources;
		DirectX::CreateDDSTextureFromMemory(
			pDevice.Get(),
			batch,
			pBuffer,
			fileSize,
			&pTexture
		);
		auto textureDesc = pTexture->GetDesc();

		batch.End(pDirectX->GetCommandQueue().Get());
#endif
		// Describe and create a SRV for the texture.
		D3D12_SHADER_RESOURCE_VIEW_DESC srvDesc = {};
		srvDesc.Shader4ComponentMapping = D3D12_DEFAULT_SHADER_4_COMPONENT_MAPPING;
		srvDesc.Format = textureDesc.Format;
		srvDesc.ViewDimension = D3D12_SRV_DIMENSION_TEXTURE2D;
		srvDesc.Texture2D.MipLevels = textureDesc.MipLevels;
		pDevice->CreateShaderResourceView(pTexture.Get(), &srvDesc, pHeap->GetCPUDescriptorHandleForHeapStart());

		strcpy(pInfo->filename, pFileName);
		pInfo->count = 1;
		pInfo->texData.width = textureDesc.Width;
		pInfo->texData.height = textureDesc.Height;
		pInfo->texData.srcWidth = textureDesc.Width;
		pInfo->texData.srcHeight = textureDesc.Height;
		pInfo->texData.pTexture.Swap(pTexture);

		free(pBuffer);


		return &pInfo->texData;
	}
	return NULL;
}

// eNX`[̉
void ReleaseTexture(ComPtr<ID3D12Resource>& pTexture)
{
	if( pTexture == NULL ) return;

	for( int i=0; i < TEXTURE_MAX; i++ )
	{
		LoadedTextureInfo* pInfo = &g_loadedTextures[i];
		if( pInfo->count == 0 ) continue;

		if( pInfo->texData.pTexture == pTexture )
		{
			pInfo->count --;
			if( pInfo->count == 0 )
			{
				ZeroMemory(pInfo, sizeof(*pInfo));

				pInfo->texData.pTexture.Reset();
				pTexture.Reset();
			}
		}
	}
}

Texture::Texture(void)
	: m_width(0)
	, m_height(0)
	, m_srcWidth(0)
	, m_srcHeight(0)
{
}

Texture::~Texture(void)
{
	Finalize();
}

// eNX`[̓ǂݍ
bool Texture::Load(const char* pFileName)
{
	const TextureData* pData = LoadTexture(pFileName);
	if( pData == NULL ) return false;

	m_pTexture	= pData->pTexture;
	m_srcWidth	= pData->srcWidth;
	m_srcHeight	= pData->srcHeight;
	m_width		= pData->width;
	m_height	= pData->height;
	return true;
}

// eNX`[̓ǂݍ
bool Texture::CreateFromMemory(const void* pData, UINT size)
{
#if false
	IDirect3DTexture9* pTexture;
	D3DXIMAGE_INFO info;

	HRESULT hr = D3DXCreateTextureFromFileInMemoryEx(
		DirectXLib::Instance.GetDirect3DDevice(),
		pData,
		size,
		D3DX_DEFAULT,
		D3DX_DEFAULT,
		1,
		0,
		D3DFMT_A8R8G8B8,
		D3DPOOL_MANAGED,
		D3DX_FILTER_NONE,
		D3DX_DEFAULT,
		0x00000000,
		&info,
		NULL,
		&pTexture);
	if( FAILED(hr) )
	{
		return false;
	}
	D3DSURFACE_DESC desc;
	pTexture->GetLevelDesc(0, &desc);

	m_pTexture	= pTexture;
	m_srcWidth	= info.Width;
	m_srcHeight	= info.Height;
	m_width		= desc.Width;
	m_height	= desc.Height;
#endif

	return true;
}

// eNX`[̉
void Texture::Finalize(void)
{
	if( m_pTexture )
	{
		ReleaseTexture(m_pTexture);
		m_pTexture.Reset();
	}
}
