#define FBXSDK_SHARED
#include "FbxMesh.h"
#include "DrawBuffer.h"
#include "Texture.h"
#include "3DHelper.h"
#include <stdio.h>


#define	SAFE_DESTROY(x)	{ if(x) { (x)->Destroy(); (x) = NULL; } }

extern bool LoadMesh(MeshData* pMeshData, FbxMesh* pMesh);
extern void LoadMaterial(MaterialData* pMaterialData, FbxMesh* pMesh);


static char RootPath[MAX_PATH];

// }eȀ
extern void InitMaterial(MaterialData* pMaterial, int materialNum);

// FBX̓ǂݍ
FBXMeshData LoadFBXMesh(const char* pFileName)
{
	FBXMeshData fbxMeshData;
	ZeroMemory(&fbxMeshData, sizeof(fbxMeshData));

	strcpy_s(RootPath, pFileName);
	int i;
	for( i = strlen(RootPath); 0 < i; i-- )
	{
		if( RootPath[i] == '/' ) break;
	}
	RootPath[i] = '\0';

	// FBX̃}l[W[ƃC|[^[̐
	FbxManager* pManager = FbxManager::Create();
	FbxScene* pScene = FbxScene::Create(pManager, "");

	// t@C̃C|[g
	FbxImporter* pImpoter = FbxImporter::Create(pManager, "");
	pImpoter->Initialize(pFileName);
	pImpoter->Import(pScene);
	SAFE_DESTROY(pImpoter);


	fbxsdk::FbxGeometryConverter converter(pManager);
	// }eA1bVɕ
	converter.SplitMeshesPerMaterial(pScene, true);
	// SĂOp`ɂ
	converter.Triangulate(pScene, true);

	// bṼobt@m
	int meshNum = pScene->GetSrcObjectCount<FbxMesh>();
	MeshData* pMeshData = (MeshData*)malloc(meshNum * sizeof(MeshData));
	ZeroMemory(pMeshData, meshNum * sizeof(MeshData));

	// }eÃobt@m
	int materialNum = meshNum;
	MaterialData* pMaterialData = (MaterialData*)malloc(materialNum * sizeof(MaterialData));
	ZeroMemory(pMaterialData, materialNum * sizeof(MaterialData));


	D3DXMatrixIdentity(&fbxMeshData.Model.world);
	fbxMeshData.Model.meshNum		= meshNum;
	fbxMeshData.Model.pMesh			= pMeshData;
	fbxMeshData.Model.materialNum	= materialNum;
	fbxMeshData.Model.pMaterial		= pMaterialData;

	// bVPʂœWJĂ
	for( int i=0; i < meshNum; i++ )
	{
		FbxMesh* pMesh = pScene->GetSrcObject<FbxMesh>(i);

		LoadMesh(&pMeshData[i], pMesh);
		LoadMaterial(&pMaterialData[i], pMesh);

		pMeshData[i].materialIndex = i;
	}

	SAFE_DESTROY(pScene);
	SAFE_DESTROY(pManager);

	return fbxMeshData;
}

// bV̉
void ReleaseFbxMesh(FBXMeshData* pData)
{
	if( !pData ) return;

	ReleaseModel(&pData->Model);
}

// FBXbV̕`
void RenderFbxMesh(FBXMeshData* pData)
{
	if( !pData ) return;

	RenderModel(&pData->Model);
}

extern void LoadIndeces(MeshData* pMeshData, FbxMesh* pMesh);
extern void LoadPosition(MeshData* pMeshData, FbxMesh* pMesh);
extern void LoadNormal(MeshData* pMeshData, FbxMesh* pMesh);
extern void LoadUV(MeshData* pMeshData, FbxMesh* pMesh);
extern void LoadColor(MeshData* pMeshData, FbxMesh* pMesh);

// FBX̉
bool LoadMesh(MeshData* pMeshData, FbxMesh* pMesh)
{
	LoadIndeces(pMeshData, pMesh);

	LoadPosition(pMeshData, pMesh);
	LoadUV(pMeshData, pMesh);
	LoadColor(pMeshData, pMesh);
	LoadNormal(pMeshData, pMesh);

	return true;
}


void LoadIndeces(MeshData* pMeshData, FbxMesh* pMesh)
{
	int polyCount = pMesh->GetPolygonCount();
	UINT size = (UINT)((polyCount * 3) * sizeof(UINT16));

	pMeshData->fvf		= FVF_MESH_TEX_VERTEX;
	pMeshData->primType	= D3DPT_TRIANGLELIST;
	pMeshData->primNum	= (UINT)polyCount;
	pMeshData->indexNum = (UINT)(polyCount * 3);
	pMeshData->pIndexBuffer = CreateIndexBuffer(NULL, size);

	UINT16* pIndeces;
	// obt@bNăf[^
	pMeshData->pIndexBuffer->Lock(0, size, (void**)&pIndeces, 0);

	for( int polyIdx=0; polyIdx < polyCount; polyIdx++ )
	{
		pIndeces[polyIdx * 3 + 0] = polyIdx * 3 + 2;
		pIndeces[polyIdx * 3 + 1] = polyIdx * 3 + 1;
		pIndeces[polyIdx * 3 + 2] = polyIdx * 3 + 0;
	}
	pMeshData->pIndexBuffer->Unlock();
}

void LoadPosition(MeshData* pMeshData, FbxMesh* pMesh)
{
	int vertexNum = pMesh->GetPolygonVertexCount();
	UINT size = (UINT)(vertexNum * sizeof(MeshVertex));

	FbxVector4* source = pMesh->GetControlPoints();
	// bṼgXtH[
	FbxVector4 T = pMesh->GetNode()->GetGeometricTranslation(FbxNode::eSourcePivot);
	FbxVector4 R = pMesh->GetNode()->GetGeometricRotation(FbxNode::eSourcePivot);
	FbxVector4 S = pMesh->GetNode()->GetGeometricScaling(FbxNode::eSourcePivot);
	FbxAMatrix TRS = FbxAMatrix(T, R, S);
	//	S_ϊ
	for( int v = 0; v < pMesh->GetControlPointsCount(); v++ )
	{
		source[v] = TRS.MultT(source[v]);
	}

	pMeshData->vertexNum = (UINT)vertexNum;
	pMeshData->vertexStride = sizeof(MeshVertex);
	pMeshData->pVertexBuffer = CreateVertexBuffer(NULL, size);

	MeshVertex* pVertices;
	// obt@bNăf[^
	pMeshData->pVertexBuffer->Lock(0, size, (void**)&pVertices, 0);

	int* pIndex = pMesh->GetPolygonVertices();
	for( int vtxIdx=0; vtxIdx < vertexNum; vtxIdx++ )
	{
		int index = pIndex[vtxIdx];
		FbxVector4& pos = source[index];

		pVertices[vtxIdx].x		=-(float)pos[0];
		pVertices[vtxIdx].y		= (float)pos[1];
		pVertices[vtxIdx].z		= (float)pos[2];
		pVertices[vtxIdx].nx	= 0.0f;
		pVertices[vtxIdx].ny	= 1.0f;
		pVertices[vtxIdx].nz	= 0.0f;
		pVertices[vtxIdx].u		= 0.0f;
		pVertices[vtxIdx].v		= 0.0f;
		pVertices[vtxIdx].color	= 0xffffff;
	}
	pMeshData->pVertexBuffer->Unlock();
}

void LoadNormal(MeshData* pMeshData, FbxMesh* pMesh)
{
	FbxArray<FbxVector4> normals;
	pMesh->GetPolygonVertexNormals(normals);

	UINT size = pMeshData->vertexNum * sizeof(MeshVertex);

	MeshVertex* pVertices;
	// obt@bNăf[^
	pMeshData->pVertexBuffer->Lock(0, size, (void**)&pVertices, 0);

	for( int vtxIdx=0; vtxIdx < normals.Size(); vtxIdx++ )
	{
		FbxVector4& normal = normals[vtxIdx];

		pVertices[vtxIdx].nx	=-(float)normal[0];
		pVertices[vtxIdx].ny	= (float)normal[1];
		pVertices[vtxIdx].nz	= (float)normal[2];
	}
	pMeshData->pVertexBuffer->Unlock();
}

void LoadUV(MeshData* pMeshData, FbxMesh* pMesh)
{
	FbxStringList uvSetNames;
	pMesh->GetUVSetNames(uvSetNames);

	FbxArray<FbxVector2> uvSets;
	pMesh->GetPolygonVertexUVs(uvSetNames.GetStringAt(0), uvSets);

	UINT size = pMeshData->vertexNum * sizeof(MeshVertex);

	MeshVertex* pVertices;
	// obt@bNăf[^
	pMeshData->pVertexBuffer->Lock(0, size, (void**)&pVertices, 0);

	for( int vtxIdx=0; vtxIdx < uvSets.Size(); vtxIdx++ )
	{
		FbxVector2& uvSet = uvSets[vtxIdx];

		pVertices[vtxIdx].u	= (float)uvSet[0];
		pVertices[vtxIdx].v	= (float)(1.0 - uvSet[1]);
	}
	pMeshData->pVertexBuffer->Unlock();
}

void LoadColor(MeshData* pMeshData, FbxMesh* pMesh)
{
	int colorElementCount = pMesh->GetElementVertexColorCount();
	if( colorElementCount == 0 ) return;

	FbxGeometryElementVertexColor* pColor = pMesh->GetElementVertexColor(0);

	FbxLayerElement::EMappingMode mapMode = pColor->GetMappingMode();
	FbxLayerElement::EReferenceMode refMode = pColor->GetReferenceMode();

	if( mapMode == FbxLayerElement::eByPolygonVertex )
	{
		if( refMode == FbxLayerElement::eIndexToDirect )
		{
			UINT size = pMeshData->vertexNum * sizeof(MeshVertex);
			MeshVertex* pVertices;
			// obt@bNăf[^
			pMeshData->pVertexBuffer->Lock(0, size, (void**)&pVertices, 0);

			FbxLayerElementArrayTemplate<FbxColor>& colors = pColor->GetDirectArray();
			FbxLayerElementArrayTemplate<int>& indeces = pColor->GetIndexArray();
			for( int i=0; i < indeces.GetCount(); i++ )
			{
				FbxColor& color = colors.GetAt(indeces.GetAt(i));
				DWORD a = (DWORD)(color.mAlpha * 255.0);
				DWORD r = (DWORD)(color.mRed   * 255.0);
				DWORD g = (DWORD)(color.mGreen * 255.0);
				DWORD b = (DWORD)(color.mBlue  * 255.0);

				pVertices[i].color = (a << 24) + (r << 16) + (g << 8) + (b);
			}
			pMeshData->pVertexBuffer->Unlock();
		}
	}
}


void LoadMaterial(MaterialData* pMaterialData, FbxMesh* pMesh)
{
	InitMaterial(pMaterialData, 1);

	FbxLayerElementMaterial* pElementMaterial = pMesh->GetElementMaterial();
	if( pElementMaterial )
	{
		// }eA
		int index = pElementMaterial->GetIndexArray().GetAt(0);

		FbxSurfaceMaterial* pMaterial = pMesh->GetNode()->GetSrcObject<FbxSurfaceMaterial>(index);
		if( pMaterial )
		{
			FbxProperty prop = pMaterial->FindProperty(FbxSurfaceMaterial::sDiffuse);
			//FbxFileTexture* pFileTexture = prop.GetSrcObject<FbxFileTexture>();

			//	eNX`ǂݍ
			const char* filename = NULL;
			int fileTextureCount = prop.GetSrcObjectCount<FbxFileTexture>();
			if (fileTextureCount > 0) {
				FbxFileTexture* FileTex = prop.GetSrcObject<FbxFileTexture>(0);
				filename = FileTex->GetFileName();
			} else {
				int numLayer = prop.GetSrcObjectCount<FbxLayeredTexture>();
				if (numLayer > 0) {
					FbxLayeredTexture* LayerTex = prop.GetSrcObject<FbxLayeredTexture>(0);
					FbxFileTexture* FileTex = LayerTex->GetSrcObject<FbxFileTexture>(0);
					filename = FileTex->GetFileName();
				}
			}
			if( filename == NULL ) return;

			//	eNX`[̎擾
			char* pFileName;
			size_t size = 0;
			FbxUTF8ToAnsi(filename, pFileName, &size);

			// PSDgpĂꍇ͓ǂ߂Ȃ̂TGAŎ
			char* ext = (char*)strstr(pFileName, ".psd");
			if( ext )
			{
				strcpy(ext, ".tga");
			}

			char path[MAX_PATH];
			strcpy_s(path, RootPath);
			strcat_s(path, "/texture/");
			strcat_s(path, pFileName);
			LoadTexture(path, &pMaterialData->textureData);

			FbxFree(pFileName);
		}
	}
}

