#include "DirectXLib/DirectXLib.h"
#include "3DSampleScene.h"
#include "SystemScene.h"


Sample3DScene::Sample3DScene(void)
{
}
Sample3DScene::~Sample3DScene(void)
{
}

// 
bool Sample3DScene::Initialize(void)
{
	// 3̖󃂃f𐶐
	if( !m_model[0].Create3DAxis(true, true, true) )
	{
		return false;
	}
	if( !m_model[1].CreateTriangle() )
	{
		return false;
	}
	if( !m_model[2].CreateCamera() )
	{
		return false;
	}
	if( !m_model[3].CreateViewFrustum() )
	{
		return false;
	}
	return true;
}

// 
void Sample3DScene::Finalize(void)
{
	for( int i=0; i < ARRAYSIZE(m_model); i++ )
	{
		m_model[i].Finalize();
	}
}

// XV
void Sample3DScene::Update(void)
{
	D3DXMatrixPerspectiveFovLH(
		&m_projectionMatrix,
		D3DXToRadian(45.0f),
		640.0f / 480.0f,
		0.01f,
		1000.0f
	);

	D3DXVECTOR3 cameraPos(0.0f, 0.75f, -0.0f);
	D3DXVECTOR3 targetPos(0.0f, 0.0f, 0.0f);
	D3DXVECTOR3 upVec(0.0f, 0.0f, 1.0f);
	cameraPos = D3DXVECTOR3(0.0f, 0.1f, -0.75f);
	targetPos = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	D3DXMatrixLookAtLH(&m_viewMatrix, &cameraPos, &targetPos, &upVec);

	for( int i=0; i < ARRAYSIZE(m_model); i++ )
	{
		D3DXVECTOR3 pos(0.0f, 0.0f, 0.0f);
		D3DXVECTOR3 scaleRate(1.0f, 1.0f, 1.0f);
		D3DXVECTOR3 angle(0.0f, 0.0f, 0.0f);
		switch( i )
		{
		case 0:
			angle.z = D3DXToRadian(0.0f);
			pos.y = -0.1f;
			break;
		case 1:
		{
			float scaleBase = 0.025f * 1.25f;
			scaleRate = D3DXVECTOR3(scaleBase, scaleBase, scaleBase);
			angle.y = D3DXToRadian(45.0f);
			pos.x = 0.1f; pos.z = 0.1f;
			pos.x -= 0.1f; pos.z -= -0.05f;
		} break;
		case 2:
		{
			float scaleBase = 0.025f;
			scaleRate = D3DXVECTOR3(scaleBase, scaleBase, scaleBase);
			angle.y = D3DXToRadian(90.0f);
			//pos.x = 0.1f; pos.z = -0.05f;
		} break;
		case 3:
		{
			float scaleBase = 0.025f;
			scaleRate = D3DXVECTOR3(scaleBase, scaleBase * 0.75f, scaleBase);
			angle.y = D3DXToRadian(90.0f);
			pos.x = 0.125f;
		} break;
		}
		D3DXMATRIX trans, scale, rot;
		D3DXMatrixRotationYawPitchRoll(&rot, angle.y, angle.x, angle.z);
		D3DXMatrixTranslation(&trans, pos.x, pos.y, pos.z);
		D3DXMatrixScaling(&scale, scaleRate.x, scaleRate.y, scaleRate.z);

		D3DXMATRIX world;
		D3DXMatrixIdentity(&world);
		D3DXMatrixMultiply(&world, &world, &scale);
		D3DXMatrixMultiply(&world, &world, &rot);
		D3DXMatrixMultiply(m_model[i].GetWorld(), &world, &trans);
		if( i == 1 )
		{
			angle.y = -D3DXToRadian(-15.0f);
			D3DXMatrixRotationY(&rot, angle.y);
			D3DXMatrixMultiply(m_model[i].GetWorld(), m_model[i].GetWorld(), &rot);
		}
	}

	if( INPUT->IsKeyPush(Input::A) )
	{
		SCENEMANAGER->JumpScene(new SystemScene);
	}
}

// `
void Sample3DScene::Render(void)
{
	IDirect3DDevice9* pDevice = DirectXLib::Instance.GetDirect3DDevice();
	DirectX* pDirectX = DirectXLib::Instance.GetDirectX();

	pDirectX->ClearBackBuffer(D3DCLEAR_TARGET, 0xffffffff, 0.0, 0);

	// View, Projections̐ݒ
	pDevice->SetTransform(D3DTS_VIEW, &m_viewMatrix);
	pDevice->SetTransform(D3DTS_PROJECTION, &m_projectionMatrix);

	// ʏ폑݃[hɐݒ
	pDirectX->SetRenderMode(DirectX::Normal, true);

	pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	pDevice->SetRenderState(D3DRS_ZFUNC, D3DCMP_LESSEQUAL);

	for( int i=0; i < ARRAYSIZE(m_model); i++ )
	{
		m_model[i].Render();
	}
}