#include "DirectXLib/DirectXLib.h"
#include "FbxTestScene.h"
#include "Camera.h"
#include "SystemScene.h"



FbxTestScene::FbxTestScene(void)
	: m_playerPos(0.0f, 3.0f, 0.0f)
{
}
FbxTestScene::~FbxTestScene(void)
{
}

// 
bool FbxTestScene::Initialize(void)
{	
	m_fbxModel.Load("assets/stage/stage2_1.fbx");
	m_player.Load("assets/zunko_model_data/TouhokuZunko_FromBlender_20140620_3.fbx");
	m_shpere.CreateShpere(32);

	InitializeCamera(D3DXVECTOR3(0.0f, 1.5f, 2.0f), m_playerPos);
	SetCameraSpeed(0.08f);

	m_playerRadius = 0.1f;
	m_playerSpeed = 0.1f;
	return true;
}

// 
void FbxTestScene::Finalize(void)
{
	m_fbxModel.Finalize();
	m_player.Finalize();
	m_shpere.Finalize();
}


// XV
void FbxTestScene::Update(void)
{
	UpdatePlayer();
	UpdateCamera(&m_playerPos);

	// f̍sZo
	D3DXMATRIX rot, trans, scale;
	D3DXMatrixScaling(&scale, m_playerRadius, m_playerRadius, m_playerRadius);
	D3DXMatrixRotationY(&rot, D3DXToRadian(0.0f));
	D3DXMatrixTranslation(&trans, m_playerPos.x, m_playerPos.y, m_playerPos.z);
	D3DXMatrixMultiply(m_shpere.GetWorld(), &scale, &rot);
	D3DXMatrixMultiply(m_shpere.GetWorld(), m_shpere.GetWorld(), &trans);

	*m_player.GetWorld() = *m_shpere.GetWorld();
	m_player.Animate();

	const float StageScale = 1.0f;
	D3DXMatrixScaling(m_fbxModel.GetWorld(), StageScale, StageScale, StageScale);

	if( INPUT->IsKeyPush(Input::A) )
	{
		SCENEMANAGER->JumpScene(new SystemScene);
	}
}

// `
void FbxTestScene::Render(void)
{
	IDirect3DDevice9* pDevice = DirectXLib::Instance.GetDirect3DDevice();
	DirectX* pDirectX = DirectXLib::Instance.GetDirectX();

	// ʏ폑݃[hɐݒ
	pDirectX->SetRenderMode(DirectX::Normal, false);

	// ZeXg̔r@
	pDevice->SetRenderState(D3DRS_ZFUNC, D3DCMP_LESSEQUAL);

	SetLight();

	// View, Projections̐ݒ
	pDevice->SetTransform(D3DTS_VIEW, GetViewMatrix());
	pDevice->SetTransform(D3DTS_PROJECTION, GetProjectionMatrix());

	// f̕`
	m_fbxModel.Render();
	m_player.Render();
	//m_shpere.Render();

	// fobO\
	float dist = D3DXVec3Length(&(GetCameraPosition() - GetCameraTarget()));
	DEBUGFONT->DrawFormatText(0, 20, 0xffffffff, "CameraDist:%f", dist);
}

// Cg̐ݒ
void FbxTestScene::SetLight(void)
{
	IDirect3DDevice9* pDevice = DirectXLib::Instance.GetDirect3DDevice();

	D3DLIGHT9 light;
	ZeroMemory(&light, sizeof(light));
	light.Type = D3DLIGHT_DIRECTIONAL;
	light.Direction.x = 0.0f;
	light.Direction.y = -0.5f;
	light.Direction.z = 0.2f;
	light.Diffuse.a = 1.0f;
	light.Diffuse.r = 0.8f;
	light.Diffuse.g = 0.8f;
	light.Diffuse.b = 0.8f;
	light.Ambient.a = 1.0f;
	light.Ambient.r = 0.5f;
	light.Ambient.g = 0.5f;
	light.Ambient.b = 0.5f;

	// CgLɂ
	pDevice->SetRenderState(D3DRS_LIGHTING, TRUE);

	pDevice->LightEnable(0, TRUE);
	pDevice->SetLight(0, &light);
}

// vC[̍XV
void FbxTestScene::UpdatePlayer(void)
{
	// XZʏł̐iš擾
	D3DXVECTOR3 moveVec(GetCameraTarget() - GetCameraPosition()), moveSideVec;
	moveVec.y = 0.0f;

	D3DXMATRIX rot;
	// XZʏōĚ擾
	D3DXMatrixRotationY(&rot, D3DXToRadian(90));
	D3DXVec3TransformCoord(&moveSideVec, &moveVec, &rot);

	D3DXVECTOR3 move(0.0f, 0.0f, 0.0f);
	// Öړl
	if( INPUT->IsKeyPress(Input::Up) )			move += moveVec;
	else if( INPUT->IsKeyPress(Input::Down) )	move -= moveVec;
	// Ëړl
	if( INPUT->IsKeyPress(Input::Left) )		move -= moveSideVec;
	else if( INPUT->IsKeyPress(Input::Right) )	move += moveSideVec;

	// ړl̑xm_playerSpeedɍ킹
	D3DXVec3Normalize(&move, &move);
	D3DXVec3Scale(&move, &move, m_playerSpeed);

	// ړl̔f
	m_playerPos += move;


	float hitDist =  5.0f;
	D3DXVECTOR3 hitPos(m_playerPos);
	// ɓ蔻Ƃꍇ͏ォ画肷
	hitPos.y += 1.0f;
	// nʂƂ̓蔻(vC[̈ʒu^ɓ蔻s)
	if( m_fbxModel.RayPick(&hitPos, &D3DXVECTOR3(0.0f, -1.0f, 0.0f), &hitDist, &hitPos) != -1 )
	{
		// ꍇ̓vC[̍nʂɐݒu
		m_playerPos.y = hitPos.y + m_playerRadius;
	}
}
